@@ -130,6 +130,11 @@ int ncdirect_cursor_down(ncdirect* nc, int num){
   return ret;
 }
 
+static inline int
+ncdirect_cursor_down_f(ncdirect* nc, int num, fbuf* f){
+  return emit_scrolls(&nc->tcache, num, f);
+}
+
 int ncdirect_clear(ncdirect* nc){
   const char* clearscr = get_escape(&nc->tcache, ESCAPE_CLEAR);
   if(clearscr){
@@ -179,77 +184,16 @@ int ncdirect_cursor_disable(ncdirect* nc){
 }
 
 static int
-cursor_yx_get(int ttyfd, const char* u7, int* y, int* x){
+cursor_yx_get(ncdirect* n, int ttyfd, const char* u7, int* y, int* x){
+  struct inputctx* ictx = n->tcache.ictx;
+  if(ncdirect_flush(n)){
+    return -1;
+  }
   if(tty_emit(u7, ttyfd)){
     return -1;
   }
-  bool done = false;
-  enum { // what we expect now
-    CURSOR_ESC, // 27 (0x1b)
-    CURSOR_LSQUARE,
-    CURSOR_ROW, // delimited by a semicolon
-    CURSOR_COLUMN,
-    CURSOR_R,
-  } state = CURSOR_ESC;
-  int row = 0, column = 0;
-  int r;
-  char in;
-  do{
-#ifndef __MINGW64__
-    while((r = read(ttyfd, &in, 1)) == 1){
-#else
-    while((r = getc(stdin)) >= 0){ // FIXME
-      in = r;
-#endif
-      bool valid = false;
-      switch(state){
-        case CURSOR_ESC: valid = (in == NCKEY_ESC); state = CURSOR_LSQUARE; break;
-        case CURSOR_LSQUARE: valid = (in == '['); state = CURSOR_ROW; break;
-        case CURSOR_ROW:
-          if(isdigit(in)){
-            row *= 10;
-            row += in - '0';
-            valid = true;
-          }else if(in == ';'){
-            state = CURSOR_COLUMN;
-            valid = true;
-          }
-          break;
-        case CURSOR_COLUMN:
-          if(isdigit(in)){
-            column *= 10;
-            column += in - '0';
-            valid = true;
-          }else if(in == 'R'){
-            state = CURSOR_R;
-            valid = true;
-          }
-          break;
-        case CURSOR_R: default: // logical error, whoops
-          break;
-      }
-      if(!valid){
-        logerror("Unexpected result (%c, %d) from terminal\n", in, in);
-        break;
-      }
-      if(state == CURSOR_R){
-        done = true;
-        break;
-      }
-    }
-    // need to loop 0 to handle slow terminals, see for instance screen =[
-  }while(!done && (r >= 0 || (errno == EINTR || errno == EAGAIN || errno == EBUSY)));
-  if(!done){
-    logerror("Error reading cursor location\n");
-    return -1;
-  }
-  if(y){
-    *y = row;
-  }
-  if(x){
-    *x = column;
-  }
-  loginfo("cursor at y=%d x=%d\n", row, column);
+  get_cursor_location(ictx, y, x);
+  loginfo("cursor at y=%d x=%d\n", *y, *x);
   return 0;
 }
 
@@ -266,7 +210,7 @@ int ncdirect_cursor_move_yx(ncdirect* n, int y, int x){
     if(hpa){
       return term_emit(tiparm(hpa, x), n->ttyfp, false);
     }else if(n->tcache.ttyfd >= 0 && u7){
-      if(cursor_yx_get(n->tcache.ttyfd, u7, &y, NULL)){
+      if(cursor_yx_get(n, n->tcache.ttyfd, u7, &y, NULL)){
         return -1;
       }
     }else{
@@ -276,7 +220,7 @@ int ncdirect_cursor_move_yx(ncdirect* n, int y, int x){
     if(!vpa){
       return term_emit(tiparm(vpa, y), n->ttyfp, false);
     }else if(n->tcache.ttyfd >= 0 && u7){
-      if(cursor_yx_get(n->tcache.ttyfd, u7, NULL, &x)){
+      if(cursor_yx_get(n, n->tcache.ttyfd, u7, NULL, &x)){
         return -1;
       }
     }else{
@@ -295,6 +239,7 @@ int ncdirect_cursor_move_yx(ncdirect* n, int y, int x){
   return -1; // we will not be moving the cursor today
 }
 
+/*
 // an algorithm to detect inverted cursor reporting on terminals 2x2 or larger:
 //  * get initial cursor position / push cursor position
 //  * move right using cursor-independent routines
@@ -408,6 +353,7 @@ detect_cursor_inversion_wrapper(ncdirect* n, const char* u7, int* y, int* x){
   // of ttyfd, as needed by cursor interrogation.
   return detect_cursor_inversion(n, u7, toty, totx, y, x);
 }
+*/
 
 // no terminfo capability for this. dangerous--it involves writing controls to
 // the terminal, and then reading a response. many things can distupt this
@@ -444,24 +390,7 @@ int ncdirect_cursor_yx(ncdirect* n, int* y, int* x){
   if(!x){
     x = &xval;
   }
-  if(!n->tcache.detected_cursor_inversion){
-    ret = detect_cursor_inversion_wrapper(n, u7, y, x);
-  }else{
-    ret = cursor_yx_get(n->tcache.ttyfd, u7, y, x);
-  }
-  if(ret == 0){
-    if(n->tcache.inverted_cursor){
-      int tmp = *y;
-      *y = *x;
-      *x = tmp;
-    }else{
-      // we use 0-based coordinates, but known terminals use 1-based
-      // coordinates. the only known exception is kmscon, which is
-      // incidentally the only one which inverts its response.
-      --*y;
-      --*x;
-    }
-  }
+  ret = cursor_yx_get(n, n->tcache.ttyfd, u7, y, x);
   if(tcsetattr(n->tcache.ttyfd, TCSANOW, &oldtermios)){
     fprintf(stderr, "Couldn't restore terminal mode on %d (%s)\n",
             n->tcache.ttyfd, strerror(errno)); // don't return error for this
@@ -502,60 +431,173 @@ ncdirect_align(struct ncdirect* n, ncalign_e align, int c){
   return INT_MAX;
 }
 
+// y is an out-only param, indicating the location where drawing started
 static int
-ncdirect_dump_plane(ncdirect* n, const ncplane* np, int xoff){
-  const int toty = ncdirect_dim_y(n);
+ncdirect_dump_sprixel(ncdirect* n, const ncplane* np, int xoff, int* y, fbuf* f){
   int dimy, dimx;
   ncplane_dim_yx(np, &dimy, &dimx);
-  if(np->sprite){
-    int y;
-    const char* u7 = get_escape(&n->tcache, ESCAPE_U7);
-    if(cursor_yx_get(n->tcache.ttyfd, u7, &y, NULL)){
+  const int toty = ncdirect_dim_y(n);
+  // flush our FILE*, as we're about to use UNIX I/O (since we can't rely on
+  // stdio to transfer large amounts at once).
+  if(ncdirect_flush(n)){
+    return -1;
+  }
+  if(ncdirect_cursor_yx(n, y, NULL)){
+    return -1;
+  }
+  if(toty - dimy < *y){
+    int scrolls = *y - 1;
+    *y = toty - dimy;
+    if(*y < 0){
+      *y = 0;
+    }
+    scrolls -= *y;
+    // perform our scrolling outside of the fbuf framework, as we need it
+    // to happen immediately for fbcon
+    if(ncdirect_cursor_move_yx(n, *y, xoff)){
       return -1;
     }
-    if(ncdirect_cursor_move_yx(n, y, xoff)){
+    if(emit_scrolls(&n->tcache, scrolls, f) < 0){
       return -1;
     }
-    // flush our FILE*, as we're about to use UNIX I/O (since we can't rely on
-    // stdio to transfer large amounts at once).
-    if(ncdirect_flush(n)){
+  }
+  if(sprite_draw(&n->tcache, NULL, np->sprite, f, *y, xoff) < 0){
+    return -1;
+  }
+  if(sprite_commit(&n->tcache, f, np->sprite, true)){
+    return -1;
+  }
+  return 0;
+}
+
+static int
+ncdirect_set_bg_default_f(ncdirect* nc, fbuf* f){
+  if(ncdirect_bg_default_p(nc)){
+    return 0;
+  }
+  const char* esc;
+  if((esc = get_escape(&nc->tcache, ESCAPE_BGOP)) != NULL){
+    if(fbuf_emit(f, esc) < 0){
       return -1;
     }
-    fbuf f = {};
-    if(fbuf_init(&f)){
+  }else if((esc = get_escape(&nc->tcache, ESCAPE_OP)) != NULL){
+    if(fbuf_emit(f, esc) < 0){
       return -1;
     }
-    if(cursor_yx_get(n->tcache.ttyfd, u7, &y, NULL)){
-      return -1;
-    }
-    if(toty - dimy < y){
-      int scrolls = y - 1;
-      y = toty - dimy;
-      if(y < 0){
-        y = 0;
-      }
-      scrolls -= y;
-      // perform our scrolling outside of the fbuf framework, as we need it
-      // to happen immediately for fbdcon
-      if(ncdirect_cursor_move_yx(n, y, xoff)){
+    if(!ncdirect_fg_default_p(nc)){
+      if(ncdirect_set_fg_rgb_f(nc, ncchannels_fg_rgb(nc->channels), f)){
         return -1;
       }
-      const char* indn = get_escape(&n->tcache, ESCAPE_IND);
-      if(scrolls > 1 && indn){
-        if(term_emit(tiparm(indn, scrolls), stdout, true) < 0){
-          return -1;
-        }
-      }
-      while(scrolls--){
-        if(ncfputc('\v', stdout) < 0){
-          return -1;
-        }
-      }
     }
-    if(sprite_draw(&n->tcache, NULL, np->sprite, &f, y, xoff) < 0){
+  }
+  ncchannels_set_bg_default(&nc->channels);
+  return 0;
+}
+
+static int
+ncdirect_set_fg_default_f(ncdirect* nc, fbuf* f){
+  if(ncdirect_fg_default_p(nc)){
+    return 0;
+  }
+  const char* esc;
+  if((esc = get_escape(&nc->tcache, ESCAPE_FGOP)) != NULL){
+    if(fbuf_emit(f, esc) < 0){
       return -1;
     }
-    if(sprite_commit(&n->tcache, &f, np->sprite, true)){
+  }else if((esc = get_escape(&nc->tcache, ESCAPE_OP)) != NULL){
+    if(fbuf_emit(f, esc) < 0){
+      return -1;
+    }
+    if(!ncdirect_bg_default_p(nc)){
+      if(ncdirect_set_bg_rgb_f(nc, ncchannels_bg_rgb(nc->channels), f)){
+        return -1;
+      }
+    }
+  }
+  ncchannels_set_fg_default(&nc->channels);
+  return 0;
+}
+
+static int
+ncdirect_dump_cellplane(ncdirect* n, const ncplane* np, fbuf* f, int xoff){
+  int dimy, dimx;
+  ncplane_dim_yx(np, &dimy, &dimx);
+  const int toty = ncdirect_dim_y(n);
+  // save the existing style and colors
+  const bool fgdefault = ncdirect_fg_default_p(n);
+  const bool bgdefault = ncdirect_bg_default_p(n);
+  const uint32_t fgrgb = ncchannels_fg_rgb(n->channels);
+  const uint32_t bgrgb = ncchannels_bg_rgb(n->channels);
+  for(int y = 0 ; y < dimy ; ++y){
+    for(int x = 0 ; x < dimx ; ++x){
+      uint16_t stylemask;
+      uint64_t channels;
+      char* egc = ncplane_at_yx(np, y, x, &stylemask, &channels);
+      if(egc == NULL){
+        return -1;
+      }
+      if(ncchannels_fg_alpha(channels) == NCALPHA_TRANSPARENT){
+        ncdirect_set_fg_default_f(n, f);
+      }else{
+        ncdirect_set_fg_rgb_f(n, ncchannels_fg_rgb(channels), f);
+      }
+      if(ncchannels_bg_alpha(channels) == NCALPHA_TRANSPARENT){
+        ncdirect_set_bg_default_f(n, f);
+      }else{
+        ncdirect_set_bg_rgb_f(n, ncchannels_bg_rgb(channels), f);
+      }
+//fprintf(stderr, "%03d/%03d [%s] (%03dx%03d)\n", y, x, egc, dimy, dimx);
+      size_t egclen = strlen(egc);
+      if(fbuf_putn(f, egclen == 0 ? " " : egc, egclen == 0 ? 1 : egclen) < 0){
+        free(egc);
+        return -1;
+      }
+      free(egc);
+    }
+    // yes, we want to reset colors and emit an explicit new line following
+    // each line of output; this is necessary if our output is lifted out and
+    // used in something e.g. paste(1).
+    // FIXME replace with a SGR clear
+    ncdirect_set_fg_default_f(n, f);
+    ncdirect_set_bg_default_f(n, f);
+    if(fbuf_printf(f, "\n%*.*s", xoff, xoff, "") < 0){
+      return -1;
+    }
+    if(y == toty){
+      if(ncdirect_cursor_down_f(n, 1, f)){
+        return -1;
+      }
+    }
+  }
+  // restore the previous colors
+  if(fgdefault){
+    ncdirect_set_fg_default_f(n, f);
+  }else{
+    ncdirect_set_fg_rgb_f(n, fgrgb, f);
+  }
+  if(bgdefault){
+    ncdirect_set_bg_default_f(n, f);
+  }else{
+    ncdirect_set_bg_rgb_f(n, bgrgb, f);
+  }
+  return 0;
+}
+
+static int
+ncdirect_dump_plane(ncdirect* n, const ncplane* np, int xoff){
+//fprintf(stderr, "rasterizing %dx%d+%d\n", dimy, dimx, xoff);
+  if(xoff){
+    if(ncdirect_cursor_move_yx(n, -1, xoff)){
+      return -1;
+    }
+  }
+  fbuf f;
+  if(fbuf_init(&f)){
+    return -1;
+  }
+  if(np->sprite){
+    int y;
+    if(ncdirect_dump_sprixel(n, np, xoff, &y, &f)){
       fbuf_free(&f);
       return -1;
     }
@@ -567,88 +609,30 @@ ncdirect_dump_plane(ncdirect* n, const ncplane* np, int xoff){
         return -1;
       }
     }
-    int targy = y + dimy;
+    int targy = y + ncplane_dim_y(np);
+    const int toty = ncdirect_dim_y(n);
     if(targy > toty){
       targy = toty;
     }
     if(ncdirect_cursor_move_yx(n, targy, xoff)){
       return -1;
     }
-    return 0;
-  }
-//fprintf(stderr, "rasterizing %dx%d+%d\n", dimy, dimx, xoff);
-  // save the existing style and colors
-  const bool fgdefault = ncdirect_fg_default_p(n);
-  const bool bgdefault = ncdirect_bg_default_p(n);
-  const uint32_t fgrgb = ncchannels_fg_rgb(n->channels);
-  const uint32_t bgrgb = ncchannels_bg_rgb(n->channels);
-  for(int y = 0 ; y < dimy ; ++y){
-    if(xoff){
-      if(ncdirect_cursor_move_yx(n, -1, xoff)){
-        return -1;
-      }
-    }
-    for(int x = 0 ; x < dimx ; ++x){
-      uint16_t stylemask;
-      uint64_t channels;
-      char* egc = ncplane_at_yx(np, y, x, &stylemask, &channels);
-      if(egc == NULL){
-        return -1;
-      }
-      if(ncchannels_fg_alpha(channels) == NCALPHA_TRANSPARENT){
-        ncdirect_set_fg_default(n);
-      }else{
-        ncdirect_set_fg_rgb(n, ncchannels_fg_rgb(channels));
-      }
-      if(ncchannels_bg_alpha(channels) == NCALPHA_TRANSPARENT){
-        ncdirect_set_bg_default(n);
-      }else{
-        ncdirect_set_bg_rgb(n, ncchannels_bg_rgb(channels));
-      }
-//fprintf(stderr, "%03d/%03d [%s] (%03dx%03d)\n", y, x, egc, dimy, dimx);
-      if(fprintf(n->ttyfp, "%s", strlen(egc) == 0 ? " " : egc) < 0){
-        free(egc);
-        return -1;
-      }
-      free(egc);
-    }
-    // yes, we want to reset colors and emit an explicit new line following
-    // each line of output; this is necessary if our output is lifted out and
-    // used in something e.g. paste(1).
-    // FIXME replace with a SGR clear
-    ncdirect_set_fg_default(n);
-    ncdirect_set_bg_default(n);
-    if(ncfputc('\n', n->ttyfp) == EOF){
+  }else{
+    if(ncdirect_dump_cellplane(n, np, &f, xoff)){
+      fbuf_free(&f);
       return -1;
     }
-    if(y == toty){
-      if(ncdirect_cursor_down(n, 1)){
-        return -1;
-      }
+    if(fbuf_finalize(&f, n->ttyfp)){
+      return -1;
     }
   }
-  // restore the previous colors
-  if(fgdefault){
-    ncdirect_set_fg_default(n);
-  }else{
-    ncdirect_set_fg_rgb(n, fgrgb);
-  }
-  if(bgdefault){
-    ncdirect_set_bg_default(n);
-  }else{
-    ncdirect_set_bg_rgb(n, bgrgb);
-  }
   return 0;
 }
 
 int ncdirect_raster_frame(ncdirect* n, ncdirectv* ncdv, ncalign_e align){
   int lenx = ncplane_dim_x(ncdv);
   int xoff = ncdirect_align(n, align, lenx);
-  if(ncdirect_dump_plane(n, ncdv, xoff)){
-    free_plane(ncdv);
-    return -1;
-  }
-  int r = ncdirect_flush(n);
+  int r = ncdirect_dump_plane(n, ncdv, xoff);
   free_plane(ncdv);
   return r;
 }
@@ -864,18 +848,14 @@ ncdirect_stop_minimal(void* vnc){
     ret |= fbuf_finalize(&f, stdout);
   }
   if(nc->tcache.ttyfd >= 0){
-    if(nc->tcache.kittykbd){
-      ret |= tty_emit("\x1b[<u", nc->tcache.ttyfd);
-    }
+    ret |= tty_emit("\x1b[<u", nc->tcache.ttyfd);
     const char* cnorm = get_escape(&nc->tcache, ESCAPE_CNORM);
     if(cnorm && tty_emit(cnorm, nc->tcache.ttyfd)){
       ret = -1;
     }
     ret |= tcsetattr(nc->tcache.ttyfd, TCSANOW, nc->tcache.tpreserved);
-    ret |= close(nc->tcache.ttyfd);
   }
   ret |= ncdirect_flush(nc);
-  free_terminfo_cache(&nc->tcache);
   return ret;
 }
 
@@ -883,7 +863,7 @@ ncdirect* ncdirect_core_init(const char* termtype, FILE* outfp, uint64_t flags){
   if(outfp == NULL){
     outfp = stdout;
   }
-  if(flags > (NCDIRECT_OPTION_VERY_VERBOSE << 1)){ // allow them through with warning
+  if(flags > (NCDIRECT_OPTION_DRAIN_INPUT << 1)){ // allow them through with warning
     logwarn("Passed unsupported flags 0x%016jx\n", (uintmax_t)flags);
   }
   ncdirect* ret = malloc(sizeof(ncdirect));
@@ -891,6 +871,10 @@ ncdirect* ncdirect_core_init(const char* termtype, FILE* outfp, uint64_t flags){
     return ret;
   }
   memset(ret, 0, sizeof(*ret));
+  if(pthread_mutex_init(&ret->stats.lock, NULL)){
+    free(ret);
+    return NULL;
+  }
   ret->flags = flags;
   ret->ttyfp = outfp;
   if(!(flags & NCDIRECT_OPTION_INHIBIT_SETLOCALE)){
@@ -903,6 +887,7 @@ ncdirect* ncdirect_core_init(const char* termtype, FILE* outfp, uint64_t flags){
   }
   if(setup_signals(ret, (flags & NCDIRECT_OPTION_NO_QUIT_SIGHANDLERS),
                    true, ncdirect_stop_minimal)){
+    pthread_mutex_destroy(&ret->stats.lock);
     free(ret);
     return NULL;
   }
@@ -917,9 +902,10 @@ ncdirect* ncdirect_core_init(const char* termtype, FILE* outfp, uint64_t flags){
   }
   int cursor_y = -1;
   int cursor_x = -1;
-  if(interrogate_terminfo(&ret->tcache, termtype, ret->ttyfp, utf8,
-                          1, flags & NCDIRECT_OPTION_INHIBIT_CBREAK,
-                          TERMINAL_UNKNOWN, &cursor_y, &cursor_x, NULL)){
+  if(interrogate_terminfo(&ret->tcache, termtype, ret->ttyfp, utf8, 1,
+                          flags & NCDIRECT_OPTION_INHIBIT_CBREAK,
+                          0, &cursor_y, &cursor_x, &ret->stats, 0, 0,
+                          flags & NCDIRECT_OPTION_DRAIN_INPUT)){
     goto err;
   }
   if(cursor_y >= 0){
@@ -927,6 +913,7 @@ ncdirect* ncdirect_core_init(const char* termtype, FILE* outfp, uint64_t flags){
     // unaffected by any query spill (unconsumed control sequences). move
     // us back to that location, in case there was any such spillage.
     if(ncdirect_cursor_move_yx(ret, cursor_y, cursor_x)){
+      free_terminfo_cache(&ret->tcache);
       goto err;
     }
   }
@@ -942,6 +929,7 @@ err:
     (void)tcsetattr(ret->tcache.ttyfd, TCSANOW, ret->tcache.tpreserved);
   }
   drop_signals(ret);
+  pthread_mutex_destroy(&ret->stats.lock);
   free(ret);
   return NULL;
 }
@@ -950,6 +938,11 @@ int ncdirect_stop(ncdirect* nc){
   int ret = 0;
   if(nc){
     ret |= ncdirect_stop_minimal(nc);
+    free_terminfo_cache(&nc->tcache);
+    if(nc->tcache.ttyfd >= 0){
+      ret |= close(nc->tcache.ttyfd);
+    }
+    pthread_mutex_destroy(&nc->stats.lock);
     free(nc);
   }
   return ret;
@@ -1249,8 +1242,8 @@ int ncdirect_box(ncdirect* n, uint64_t ul, uint64_t ur,
   if(xlen < 2 || ylen < 2){
     return -1;
   }
-  char hl[WCHAR_MAX_UTF8BYTES + 1];
-  char vl[WCHAR_MAX_UTF8BYTES + 1];
+  char hl[MB_CUR_MAX + 1];
+  char vl[MB_CUR_MAX + 1];
   unsigned edges;
   edges = !(ctlword & NCBOXMASK_TOP) + !(ctlword & NCBOXMASK_LEFT);
   if(edges >= box_corner_needs(ctlword)){
@@ -1422,12 +1415,15 @@ int ncdirect_stream(ncdirect* n, const char* filename, ncstreamcb streamer,
       if(n->tcache.pixel_remove){
         fbuf f = {};
         fbuf_init_small(&f);
-        if(n->tcache.pixel_remove(lastid, &f) || fwrite(f.buf, f.used, 1, n->ttyfp) != 1){
+        if(n->tcache.pixel_remove(lastid, &f)){
           fbuf_free(&f);
           ncvisual_destroy(ncv);
           return -1;
         }
-        fbuf_free(&f);
+        if(fbuf_finalize(&f, n->ttyfp) < 0){
+          ncvisual_destroy(ncv);
+          return -1;
+        }
       }
     }
     streamer(ncv, vopts, NULL, curry);
