@@ -9,6 +9,7 @@ extern "C" {
 #include "builddef.h"
 #include "compat/compat.h"
 #include "notcurses/notcurses.h"
+#include "notcurses/direct.h"
 
 // KEY_EVENT is defined by both ncurses.h and wincon.h. since we don't use
 // either definition, kill it before inclusion of ncurses.h.
@@ -271,6 +272,15 @@ typedef struct nctabbed {
   nctabbed_options opts; // copied in nctabbed_create()
 } nctabbed;
 
+// various moving parts within a notcurses context (and the user) might need to
+// access the stats object, so throw a lock on it. we don't want the lock in
+// the actual structure since (a) it's usually unnecessary and (b) it breaks
+// memset() and memcpy().
+typedef struct ncsharedstats {
+  pthread_mutex_t lock;
+  ncstats s;
+} ncsharedstats;
+
 typedef struct ncdirect {
   ncpalette palette;         // 256-indexed palette can be used instead of/with RGB
   FILE* ttyfp;               // FILE* for output tty
@@ -279,6 +289,7 @@ typedef struct ncdirect {
   uint16_t stylemask;        // current styles
   bool initialized_readline; // have we initialized Readline?
   uint64_t flags;            // copied in ncdirect_init() from param
+  ncsharedstats stats;       // stats! not as broadly used as in notcurses
 } ncdirect;
 
 // Extracellular state for a cell during the render process. There is one
@@ -329,15 +340,6 @@ typedef struct ncpile {
   sprixel* sprixelcache;      // list of sprixels
 } ncpile;
 
-// various moving parts within a notcurses context (and the user) might need to
-// access the stats object, so throw a lock on it. we don't want the lock in
-// the actual structure since (a) it's usually unnecessary and (b) it breaks
-// memset() and memcpy().
-typedef struct ncsharedstats {
-  pthread_mutex_t lock;
-  ncstats s;
-} ncsharedstats;
-
 // the standard pile can be reached through ->stdplane.
 typedef struct notcurses {
   ncplane* stdplane; // standard plane, covers screen
@@ -375,7 +377,6 @@ typedef struct notcurses {
   int loglevel;
   ncpalette palette; // 256-indexed palette can be used instead of/with RGB
   bool palette_damage[NCPALETTESIZE];
-  unsigned stdio_blocking_save; // was stdio blocking at entry? restore on stop.
   uint64_t flags;  // copied from notcurses_options
 } notcurses;
 
@@ -710,15 +711,13 @@ sprite_scrub(const notcurses* n, const ncpile* p, sprixel* s){
 // returns -1 on error, or the number of bytes written.
 static inline int
 sprite_draw(const tinfo* ti, const ncpile* p, sprixel* s, fbuf* f,
-            int y, int x){
+            int yoff, int xoff){
   if(!ti->pixel_draw){
     return 0;
   }
-  int offy, offx;
-  ncplane_abs_yx(s->n, &offy, &offx);
 //sprixel_debug(s, stderr);
   logdebug("sprixel %u state %d\n", s->id, s->invalidated);
-  return ti->pixel_draw(ti, p, s, f, y + offy, x + offx);
+  return ti->pixel_draw(ti, p, s, f, yoff, xoff);
 }
 
 // precondition: s->invalidated is SPRIXEL_MOVED or SPRIXEL_INVALIDATED
@@ -733,7 +732,7 @@ sprite_redraw(notcurses* nc, const ncpile* p, sprixel* s, fbuf* f, int y, int x)
     // not emit it. we use sixel_maxy_pristine as a side channel to encode
     // this version information.
     bool noscroll = !ti->sixel_maxy_pristine;
-    return ti->pixel_move(s, f, noscroll);
+    return ti->pixel_move(s, f, noscroll, y, x);
   }else{
     if(!ti->pixel_draw){
       return 0;
@@ -1606,6 +1605,8 @@ ncdirect_bg_palindex_p(const ncdirect* nc){
   return ncchannels_bg_palindex_p(ncdirect_channels(nc));
 }
 
+int ncdirect_set_fg_rgb_f(ncdirect* nc, unsigned rgb, fbuf* f);
+int ncdirect_set_bg_rgb_f(ncdirect* nc, unsigned rgb, fbuf* f);
 int term_fg_rgb8(const tinfo* ti, fbuf* f, unsigned r, unsigned g, unsigned b);
 
 const struct blitset* lookup_blitset(const tinfo* tcache, ncblitter_e setid, bool may_degrade);
@@ -1745,6 +1746,44 @@ tty_check(int fd){
   return isatty(fd);
 }
 
+// attempt to cancel the specified thread (not an error if we can't; it might
+// have already exited), and then join it (an error here is propagated).
+static inline int
+cancel_and_join(const char* name, pthread_t tid, void** res){
+  if(pthread_cancel(tid)){
+    logerror("couldn't cancel %s thread\n", name); // tid might have died
+  }
+  if(pthread_join(tid, res)){
+    logerror("error joining %s thread\n", name);
+    return -1;
+  }
+  return 0;
+}
+
+static inline int
+emit_scrolls(const tinfo* ti, int count, fbuf* f){
+  if(count > 1){
+    const char* indn = get_escape(ti, ESCAPE_INDN);
+    if(indn){
+      if(fbuf_emit(f, tiparm(indn, count)) < 0){
+        return -1;
+      }
+      return 0;
+    }
+  }
+  const char* ind = get_escape(ti, ESCAPE_IND);
+  if(ind == NULL){
+    ind = "\v";
+  }
+  while(count > 0){
+    if(fbuf_emit(f, ind) < 0){
+      return -1;
+    }
+    --count;
+  }
+  return 0;
+}
+
 #undef ALLOC
 #undef API
 
