@@ -1,4 +1,3 @@
-#include "input.h"
 #include "linux.h"
 #include "version.h"
 #include "egcpool.h"
@@ -107,10 +106,8 @@ notcurses_stop_minimal(void* vnc){
     if(nc->tcache.tpreserved){
       ret |= tcsetattr(nc->tcache.ttyfd, TCSAFLUSH, nc->tcache.tpreserved);
     }
-    if(nc->tcache.kittykbd){
-      if(tty_emit("\x1b[<u", nc->tcache.ttyfd)){
-        ret = -1;
-      }
+    if(tty_emit("\x1b[<u", nc->tcache.ttyfd)){
+      ret = -1;
     }
     if((esc = get_escape(&nc->tcache, ESCAPE_RMCUP))){
       if(sprite_clear_all(&nc->tcache, f)){ // send this to f
@@ -196,7 +193,7 @@ char* ncplane_at_cursor(ncplane* n, uint16_t* stylemask, uint64_t* channels){
 char* ncplane_at_yx(const ncplane* n, int y, int x, uint16_t* stylemask, uint64_t* channels){
   if(y < n->leny && x < n->lenx){
     if(y >= 0 && x >= 0){
-      const cell* yx = &n->fb[nfbcellidx(n, y, x)];
+      const nccell* yx = &n->fb[nfbcellidx(n, y, x)];
       // if we're the right side of a wide glyph, we return the main glyph
       if(nccell_wide_right_p(yx)){
         return ncplane_at_yx(n, y, x - 1, stylemask, channels);
@@ -571,7 +568,6 @@ ncplane* ncplane_new_internal(notcurses* nc, ncplane* n,
 
 // create an ncplane of the specified dimensions, but do not yet place it in
 // the z-buffer. clear out all cells. this is for a wholly new context.
-// FIXME set up using resizecb rather than special-purpose from SIGWINCH
 static ncplane*
 create_initial_ncplane(notcurses* nc, int dimy, int dimx){
   ncplane_options nopts = {
@@ -580,7 +576,7 @@ create_initial_ncplane(notcurses* nc, int dimy, int dimx){
     .cols = dimx - (nc->margin_l + nc->margin_r),
     .userptr = NULL,
     .name = "std",
-    .resizecb = ncplane_resize_maximize,
+    .resizecb = NULL,
     .flags = 0,
   };
   return nc->stdplane = ncplane_new_internal(nc, NULL, &nopts);
@@ -1001,7 +997,7 @@ notcurses* notcurses_core_init(const notcurses_options* opts, FILE* outfp){
     fprintf(stderr, "Provided an illegal negative margin, refusing to start\n");
     return NULL;
   }
-  if(opts->flags >= (NCOPTION_NO_FONT_CHANGES << 1u)){
+  if(opts->flags >= (NCOPTION_DRAIN_INPUT << 1u)){
     fprintf(stderr, "Warning: unknown Notcurses options %016" PRIu64 "\n", opts->flags);
   }
   notcurses* ret = malloc(sizeof(*ret));
@@ -1093,7 +1089,9 @@ notcurses* notcurses_core_init(const notcurses_options* opts, FILE* outfp){
   if(interrogate_terminfo(&ret->tcache, opts->termtype, ret->ttyfp, utf8,
                           opts->flags & NCOPTION_NO_ALTERNATE_SCREEN, 0,
                           opts->flags & NCOPTION_NO_FONT_CHANGES,
-                          cursory, cursorx, &ret->stats)){
+                          cursory, cursorx, &ret->stats,
+                          ret->margin_l, ret->margin_t,
+                          opts->flags & NCOPTION_DRAIN_INPUT)){
     fbuf_free(&ret->rstate.f);
     pthread_mutex_destroy(&ret->pilelock);
     pthread_mutex_destroy(&ret->stats.lock);
@@ -1150,9 +1148,6 @@ notcurses* notcurses_core_init(const notcurses_options* opts, FILE* outfp){
       ncplane_cursor_move_yx(ret->stdplane, ret->rstate.logendy, ret->rstate.logendx);
     }
   }
-  if(set_fd_nonblocking(ret->tcache.input.infd, 1, &ret->stdio_blocking_save)){
-    goto err;
-  }
   if(!(opts->flags & NCOPTION_NO_ALTERNATE_SCREEN)){
     // perform an explicit clear since the alternate screen was requested
     // (smcup *might* clear, but who knows? and it might not have been
@@ -1247,7 +1242,6 @@ int notcurses_stop(notcurses* nc){
       goto_location(nc, &nc->rstate.f, targy, 0);
       fbuf_finalize(&nc->rstate.f, stdout);
     }
-    ret |= set_fd_nonblocking(nc->tcache.input.infd, nc->stdio_blocking_save, NULL);
     if(nc->stdplane){
       notcurses_drop_planes(nc);
       free_plane(nc->stdplane);
@@ -2346,9 +2340,6 @@ ncplane* ncplane_boundlist(ncplane* n){
 }
 
 void ncplane_set_resizecb(ncplane* n, int(*resizecb)(ncplane*)){
-  if(n == notcurses_stdplane(ncplane_notcurses(n))){
-    return;
-  }
   n->resizecb = resizecb;
 }
 
@@ -2682,11 +2673,11 @@ int notcurses_lex_margins(const char* op, notcurses_options* opts){
 }
 
 int notcurses_inputready_fd(notcurses* n){
-  return n->tcache.input.infd;
+  return inputready_fd(n->tcache.ictx);
 }
 
 int ncdirect_inputready_fd(ncdirect* n){
-  return n->tcache.input.infd;
+  return inputready_fd(n->tcache.ictx);
 }
 
 // FIXME speed this up, PoC
@@ -2981,13 +2972,19 @@ int ncplane_putstr_stained(struct ncplane* n, const char* gclusters){
 }
 
 int ncplane_putwstr_stained(ncplane* n, const wchar_t* gclustarr){
-  // maximum of six UTF8-encoded bytes per wchar_t
-  const size_t mbytes = (wcslen(gclustarr) * WCHAR_MAX_UTF8BYTES) + 1;
-  char* mbstr = malloc(mbytes); // need cast for c++ callers
+  mbstate_t ps = {};
+  const wchar_t** wset = &gclustarr;
+  size_t mbytes = wcsrtombs(NULL, wset, 0, &ps);
+  if(mbytes == (size_t)-1){
+    logerror("error converting wide string\n");
+    return -1;
+  }
+  ++mbytes;
+  char* mbstr = malloc(mbytes);
   if(mbstr == NULL){
     return -1;
   }
-  size_t s = wcstombs(mbstr, gclustarr, mbytes);
+  size_t s = wcsrtombs(mbstr, wset, mbytes, &ps);
   if(s == (size_t)-1){
     free(mbstr);
     return -1;
