@@ -4,6 +4,54 @@
 #include <notcurses/direct.h>
 #include "internal.h"
 
+// update for a new visual area of |rows|x|cols|, neither of which may be zero.
+// copies that area of the lastframe (damage map) which is shared between the
+// two. new areas are initialized to empty, just like a new plane. lost areas
+// have their egcpool entries purged.
+static nccell*
+restripe_lastframe(notcurses* nc, int rows, int cols){
+  assert(rows);
+  assert(cols);
+  const size_t size = sizeof(*nc->lastframe) * (rows * cols);
+  nccell* tmp = malloc(size);
+  if(tmp == NULL){
+    return NULL;
+  }
+  size_t copycols = nc->lfdimx > cols ? cols : nc->lfdimx;
+  size_t maxlinecopy = sizeof(nccell) * copycols;
+  size_t minlineset = sizeof(nccell) * cols - maxlinecopy;
+  unsigned zorch = nc->lfdimx > cols ? nc->lfdimx - cols : 0;
+  for(int y = 0 ; y < rows ; ++y){
+    if(y < nc->lfdimy){
+      if(maxlinecopy){
+        memcpy(&tmp[cols * y], &nc->lastframe[nc->lfdimx * y], maxlinecopy);
+      }
+      if(minlineset){
+        memset(&tmp[cols * y + copycols], 0, minlineset);
+      }
+      // excise any egcpool entries from the right of the new plane area
+      if(zorch){
+        for(unsigned x = copycols ; x < copycols + zorch ; ++x){
+          pool_release(&nc->pool, &nc->lastframe[fbcellidx(y, nc->lfdimx, x)]);
+        }
+      }
+    }else{
+      memset(&tmp[cols * y], 0, sizeof(nccell) * cols);
+    }
+  }
+  // excise any egcpool entries from below the new plane area
+  for(int y = rows ; y < nc->lfdimy ; ++y){
+    for(int x = 0 ; x < nc->lfdimx ; ++x){
+      pool_release(&nc->pool, &nc->lastframe[fbcellidx(y, nc->lfdimx, x)]);
+    }
+  }
+  free(nc->lastframe);
+  nc->lastframe = tmp;
+  nc->lfdimy = rows;
+  nc->lfdimx = cols;
+  return 0;
+}
+
 // Check whether the terminal geometry has changed, and if so, copy what can
 // be copied from the old lastframe. Assumes that the screen is always anchored
 // at the same origin. Initiates a resize cascade for the pile containing |pp|.
@@ -35,18 +83,9 @@ notcurses_resize_internal(ncplane* pp, int* restrict rows, int* restrict cols){
     *cols = 1;
   }
   if(*rows != n->lfdimy || *cols != n->lfdimx){
-    n->lfdimy = *rows;
-    n->lfdimx = *cols;
-    const size_t size = sizeof(*n->lastframe) * (n->lfdimy * n->lfdimx);
-    nccell* fb = realloc(n->lastframe, size);
-    if(fb == NULL){
+    if(restripe_lastframe(n, *rows, *cols)){
       return -1;
     }
-    n->lastframe = fb;
-    // FIXME more memset()tery than we need, both wasting work and wrecking
-    // damage detection for the upcoming render
-    memset(n->lastframe, 0, size);
-    egcpool_dump(&n->pool);
   }
 //fprintf(stderr, "r: %d or: %d c: %d oc: %d\n", *rows, oldrows, *cols, oldcols);
   if(*rows == oldrows && *cols == oldcols){
@@ -56,6 +95,12 @@ notcurses_resize_internal(ncplane* pp, int* restrict rows, int* restrict cols){
   pile->dimx = *cols;
   int ret = 0;
 //notcurses_debug(n, stderr);
+  // if this pile contains the standard plane, it ought be resized to match
+  // the viewing area before invoking any other resize callbacks.
+  if(ncplane_pile(notcurses_stdplane(n)) == pile){
+    ncplane_resize_maximize(notcurses_stdplane(n));
+  }
+  // now, begin a resize callback cascade on the root planes of the pile.
   for(ncplane* rootn = pile->roots ; rootn ; rootn = rootn->bnext){
     if(rootn->resizecb){
       ret |= rootn->resizecb(rootn);
@@ -848,8 +893,6 @@ clean_sprixels(notcurses* nc, ncpile* p, fbuf* f, int scrolls){
     }else if(s->invalidated == SPRIXEL_MOVED ||
              s->invalidated == SPRIXEL_UNSEEN ||
              s->invalidated == SPRIXEL_INVALIDATED){
-      int y, x;
-      ncplane_abs_yx(s->n, &y, &x);
 //fprintf(stderr, "1 MOVING BITMAP %d STATE %d AT %d/%d for %p\n", s->id, s->invalidated, y + nc->margin_t, x + nc->margin_l, s->n);
       if(s->invalidated == SPRIXEL_MOVED){
         if(p != nc->last_pile){
@@ -865,7 +908,7 @@ clean_sprixels(notcurses* nc, ncpile* p, fbuf* f, int scrolls){
         }
       }
       // otherwise it's a new pile, so we couldn't have been on-screen
-      int r = sprite_redraw(nc, p, s, f, y + nc->margin_t, x + nc->margin_l);
+      int r = sprite_redraw(nc, p, s, f, nc->margin_t, nc->margin_l);
       if(r < 0){
         return -1;
       }
@@ -943,26 +986,7 @@ rasterize_scrolls(const ncpile* p, fbuf* f){
       return -1;
     }
   }
-  if(scrolls > 1){
-    const char* indn = get_escape(&p->nc->tcache, ESCAPE_INDN);
-    if(indn){
-      if(fbuf_emit(f, tiparm(indn, scrolls)) < 0){
-        return -1;
-      }
-      return 0;
-    }
-  }
-  const char* ind = get_escape(&p->nc->tcache, ESCAPE_IND);
-  if(ind == NULL){
-    ind = "\v";
-  }
-  while(scrolls > 0){
-    if(fbuf_emit(f, ind) < 0){
-      return -1;
-    }
-    --scrolls;
-  }
-  return 0;
+  return emit_scrolls(&p->nc->tcache, scrolls, f);
 }
 
 // second sprixel pass in rasterization. by this time, all sixels are handled
@@ -1577,7 +1601,7 @@ char* notcurses_at_yx(notcurses* nc, int yoff, int xoff, uint16_t* stylemask, ui
   return egc;
 }
 
-int ncdirect_set_bg_rgb(ncdirect* nc, unsigned rgb){
+int ncdirect_set_bg_rgb_f(ncdirect* nc, unsigned rgb, fbuf* f){
   if(rgb > 0xffffffu){
     return -1;
   }
@@ -1585,22 +1609,29 @@ int ncdirect_set_bg_rgb(ncdirect* nc, unsigned rgb){
      && ncchannels_bg_rgb(nc->channels) == rgb){
     return 0;
   }
-  fbuf f = {};
-  if(fbuf_init_small(&f)){
-    return -1;
-  }
-  if(term_bg_rgb8(&nc->tcache, &f, (rgb & 0xff0000u) >> 16u, (rgb & 0xff00u) >> 8u, rgb & 0xffu)){
-    fbuf_free(&f);
-    return -1;
-  }
-  if(fbuf_finalize(&f, nc->ttyfp) < 0){
+  if(term_bg_rgb8(&nc->tcache, f, (rgb & 0xff0000u) >> 16u, (rgb & 0xff00u) >> 8u, rgb & 0xffu)){
     return -1;
   }
   ncchannels_set_bg_rgb(&nc->channels, rgb);
   return 0;
 }
 
-int ncdirect_set_fg_rgb(ncdirect* nc, unsigned rgb){
+int ncdirect_set_bg_rgb(ncdirect* nc, unsigned rgb){
+  fbuf f = {};
+  if(fbuf_init_small(&f)){
+    return -1;
+  }
+  if(ncdirect_set_bg_rgb_f(nc, rgb, &f)){
+    fbuf_free(&f);
+    return -1;
+  }
+  if(fbuf_finalize(&f, nc->ttyfp) < 0){
+    return -1;
+  }
+  return 0;
+}
+
+int ncdirect_set_fg_rgb_f(ncdirect* nc, unsigned rgb, fbuf* f){
   if(rgb > 0xffffffu){
     return -1;
   }
@@ -1608,18 +1639,25 @@ int ncdirect_set_fg_rgb(ncdirect* nc, unsigned rgb){
      && ncchannels_fg_rgb(nc->channels) == rgb){
     return 0;
   }
+  if(term_fg_rgb8(&nc->tcache, f, (rgb & 0xff0000u) >> 16u, (rgb & 0xff00u) >> 8u, rgb & 0xffu)){
+    return -1;
+  }
+  ncchannels_set_fg_rgb(&nc->channels, rgb);
+  return 0;
+}
+
+int ncdirect_set_fg_rgb(ncdirect* nc, unsigned rgb){
   fbuf f = {};
   if(fbuf_init_small(&f)){
     return -1;
   }
-  if(term_fg_rgb8(&nc->tcache, &f, (rgb & 0xff0000u) >> 16u, (rgb & 0xff00u) >> 8u, rgb & 0xffu)){
+  if(ncdirect_set_fg_rgb_f(nc, rgb, &f)){
     fbuf_free(&f);
     return -1;
   }
   if(fbuf_finalize(&f, nc->ttyfp) < 0){
     return -1;
   }
-  ncchannels_set_fg_rgb(&nc->channels, rgb);
   return 0;
 }
 
