@@ -174,10 +174,10 @@ int fbcon_rebuild(sprixel* s, int ycell, int xcell, uint8_t* auxvec);
 int kitty_rebuild_animation(sprixel* s, int ycell, int xcell, uint8_t* auxvec);
 int kitty_rebuild_selfref(sprixel* s, int ycell, int xcell, uint8_t* auxvec);
 int sixel_draw(const tinfo* ti, const struct ncpile *p, sprixel* s,
-               fbuf* f, int y, int x);
+               fbuf* f, int yoff, int xoff);
 int kitty_draw(const tinfo* ti, const struct ncpile *p, sprixel* s,
-               fbuf* f, int y, int x);
-int kitty_move(sprixel* s, fbuf* f, unsigned noscroll);
+               fbuf* f, int yoff, int xoff);
+int kitty_move(sprixel* s, fbuf* f, unsigned noscroll, int yoff, int xoff);
 int sixel_scrub(const struct ncpile* p, sprixel* s);
 int kitty_scrub(const struct ncpile* p, sprixel* s);
 int fbcon_scrub(const struct ncpile* p, sprixel* s);
@@ -200,7 +200,7 @@ int kitty_blit_selfref(struct ncplane* nc, int linesize, const void* data,
                        int leny, int lenx, const struct blitterargs* bargs);
 int fbcon_blit(struct ncplane* nc, int linesize, const void* data,
                int leny, int lenx, const struct blitterargs* bargs);
-int fbcon_draw(const tinfo* ti, sprixel* s, int y, int x);
+int fbcon_draw(const tinfo* ti, sprixel* s, int yoff, int xoff);
 void fbcon_scroll(const struct ncpile* p, tinfo* ti, int rows);
 void sixel_refresh(const struct ncpile* p, sprixel* s);
 
@@ -208,22 +208,6 @@ void sixel_refresh(const struct ncpile* p, sprixel* s);
 int sprixel_load(sprixel* spx, fbuf* f, int pixy, int pixx, int parse_start,
                  sprixel_e state);
 
-typedef enum {
-  // C=1 (disabling scrolling) was only introduced in 0.20.0, at the same
-  // time as animation. prior to this, graphics had to be entirely redrawn
-  // on any change, and it wasn't possible to use the bottom line.
-  KITTY_ALWAYS_SCROLLS,
-  // until 0.22.0's introduction of 'a=c' for self-referential composition, we
-  // had to keep a complete copy of the RGBA data, in case a wiped cell needed
-  // to be rebuilt. we'd otherwise have to unpack the glyph and store it into
-  // the auxvec on the fly.
-  KITTY_ANIMATION,
-  // with 0.22.0, we only ever write transparent cells after writing the
-  // original image (which we now deflate, since we needn't unpack it later).
-  // the only data we need keep is the auxvecs.
-  KITTY_SELFREF,
-} kitty_graphics_e;
-
 #ifdef __cplusplus
 }
 #endif
